import json
import logging
import os

from caroline.errors import CarolineConfigurationError
from caroline.errors import CarolineConnectionError

log = logging.getLogger(__name__)


class json_db(object):
    name = "JSON simple DB"

    def __init__(self, path):
        self.path = path
        if os.path.exists(path):
            with open(path) as f:
                try:
                    self.data = json.load(f)
                except json.decoder.JSONDecodeError:
                    # It looks like the file is there, but it's empty.
                    self.data = {}
        else:
            self.data = {}

    def load(self, scope, key):
        result = self.data.get(scope.db_key.format(key))
        if not result:
            log.debug("Key {} not found, returning None.".format(key))
            return None

        return result

    def save(self, scope):
        self.data[scope.db_key.format(scope.record_id)] = scope.data
        with open(self.path, "w") as f:
            f.write(json.dumps(self.data, indent=2, sort_keys=True))

    def all_keys(self, scope):
        return self.data.keys()



import os

from addict import Dict

config = Dict()

config.default_db = os.environ.get("CAROLINE_DEFAULT_DB", "elasticsearch")
config.connection_timeout = os.environ.get("CAROLINE_CONNECTION_TIMEOUT", 0.01)

config.redis_default_url = "redis://localhost:6379/0"

# TODO: accept additional config vars for elasticsearch besides just URL
config.elasticsearch_default_url = "localhost:9200"


redis_env_addr = os.environ.get("CAROLINE_REDIS_URL", config.redis_default_url)
es_env_addr = os.environ.get(
    "CAROLINE_ELASTICSEARCH_URL", config.elasticsearch_default_url
)



from unittest.mock import MagicMock
from unittest.mock import patch

# noinspection PyUnresolvedReferences
import elasticsearch
import pytest

# noinspection PyUnresolvedReferences
import redis

from caroline.engine import Base
from caroline.errors import CarolineConfigurationError


@patch("caroline.engine.Base._load", return_value={"hello": "world"})
def test_generic_launch(a):
    class x(Base):
        redis_conn = MagicMock()
        default = {}

    y = x("asdf")
    assert y.to_dict() == {"hello": "world"}


@patch("caroline.engine.Base._load", return_value=None)
def test_default_loading(a):
    class x(Base):
        redis_conn = MagicMock()
        default = {"yo": "world"}

    y = x("asdf")
    # asdf didn't come back with anything because _load failed, so it should
    # be the default structure.
    assert y.to_dict() == {"yo": "world"}


@patch("caroline.engine.Base._load", return_value=None)
def test_redis_key_var(a):
    class x(Base):
        redis_conn = MagicMock()
        default = {"yo": "world"}
        db_key = "snarfleblat"

    y = x("asdf")

    assert y.db_key == "::snarfleblat::{}"
    assert y.db_key_unformatted == "snarfleblat"


@patch("caroline.engine.Base._load", return_value=None)
def test_update_methods(a):
    class x(Base):
        redis_conn = MagicMock()
        default = {"yo": "world"}

    y = x("asdf")

    assert y.to_dict() == {"yo": "world"}
    y.update("yo", "hello there")
    assert y.to_dict() == {"yo": "hello there"}
    y["yo"] = "general kenobi"
    assert y.to_dict() == {"yo": "general kenobi"}


@patch("caroline.engine.Base._load", return_value=None)
def test_optional_default(a):
    class x(Base):
        redis_conn = MagicMock()

    y = x("asdf")
    assert y.default == {}


def test_multiple_dbs_configured():
    with pytest.raises(CarolineConfigurationError):

        class x(Base):
            redis_conn = MagicMock()
            elasticsearch_conn = MagicMock()

        y = x("asdf")



import json
import logging

import redis

from caroline.config import redis_env_addr
from caroline.errors import CarolineConfigurationError
from caroline.errors import CarolineConnectionError

log = logging.getLogger(__name__)

# we don't actually connect to redis until we use the object; putting it here
# means that we have the connection pool available if we need it.
# Let's see if we have a Redis environment variable set!
pool = redis.ConnectionPool.from_url(redis_env_addr)


class redis_db(object):
    name = "Redis"

    def __init__(self, redis_conn=None):
        try:
            if redis_conn:
                # We have something -- we'll run it through the same testing code to
                # make sure that it works.
                self.r = redis_conn
            else:
                self.r = redis.Redis(connection_pool=pool)
            self.r.ping()
        except redis.exceptions.ConnectionError:
            raise CarolineConnectionError("Unable to reach Redis.")
        except Exception as e:
            raise CarolineConfigurationError(
                "Caught {} -- please pass in an instantiated Redis "
                "connection.".format(e)
            )

    def load(self, scope, key):
        """
        :return: Dict or None; the loaded information from Redis.
        """
        result = self.r.get(scope.db_key.format(key))
        if not result:
            log.debug("Redis key {} not found, returning None.".format(key))
            return None

        return json.loads(result.decode())

    def save(self, scope):
        self.r.set(scope.db_key.format(scope.record_id), json.dumps(scope.data))

    def all_keys(self, scope):
        return self.r.scan_iter("{}*".format(scope.db_key))



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['caroline', 'caroline.databases']

package_data = \
{'': ['*']}

install_requires = \
['addict>=2.2,<3.0',
 'elasticsearch>=6.3,<7.0',
 'jsonschema>=2.6,<3.0',
 'redis>=2.10,<3.0']

extras_require = \
{'ci': ['codacy-coverage>=1.3,<2.0', 'pytest-cov>=2.6,<3.0']}

setup_kwargs = {
    'name': 'caroline',
    'version': '0.5.2',
    'description': 'A key/value-based JSON ODM with a memorable name.',
    'long_description': '# caroline\n\n[![Build Status](https://travis-ci.org/GrafeasGroup/caroline.svg?branch=master)](https://travis-ci.org/GrafeasGroup/caroline)\n[![Codacy Badge](https://api.codacy.com/project/badge/Grade/84632bae1d3f4dd8ad69cf90fd0a8d6b)](https://www.codacy.com/app/joe-kaufeld/caroline?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=GrafeasGroup/caroline&amp;utm_campaign=Badge_Grade)\n[![Codacy Badge](https://api.codacy.com/project/badge/Coverage/84632bae1d3f4dd8ad69cf90fd0a8d6b)](https://www.codacy.com/app/joe-kaufeld/charlotte?utm_source=github.com&utm_medium=referral&utm_content=GrafeasGroup/caroline&utm_campaign=Badge_Coverage)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n[![Packaged with Poetry](https://img.shields.io/badge/packaged%20with-poetry-blue.svg)](https://poetry.eustace.io)\n\nA key-value JSON ODM with a memorable name.\n\n### What is caroline?\nCaroline is yet another way to store JSON data. It contains backends for Elasticsearch and Redis, and allows you to use both in the same project! Included in the box (and heavily recommended that you use) is jsonschema support.\n\n\n### Why?\nThe existing options that are available didn\'t quite work for what we were looking for, so we wrote our own for internal use and decided to open source it. The goal of the project is easy integration without high overhead.\n\n### How does it work?\n\nThe only thing you need to do to get started is `from caroline import Prototype`. Then you can start building your data classes:\n\n```python\nfrom caroline import Prototype\n\nclass Dog(Prototype):\n    default = {\n        "breed": "",\n        "age": 1,\n        "name": ""\n    }\n    schema = {\n        "type": "object",\n        "properties": {\n            "breed": {\n                "type": "string"\n            },\n            "age": {\n                "type": "number"\n            },\n            "name": {\n                "type": "string"\n            }\n        }\n    }\n    \nsam = Dog(\'sam\')\n\nprint(sam)\n# >>> {"breed": "", "age": 1, "name": ""}\n\nsam.update(\'name\', \'Sam\')\n# OR\nsam[\'name\'] = \'Sam\'\n\nsam.update(\'age\', 5)\nsam.update(\'breed\', \'mutt\')\nsam.save()\n```\nWhen you create a new instance of your class with an ID (like `Dog(\'sam\')`), that\'s the key that the particular record will be saved under. That means that if you create a class with that name, it\'ll load the same record again:\n\n```python\nfrom example_above import Dog\n\nx = Dog(\'sam\')\n\nprint(x)\n# >>> {\'breed\': \'mutt\', \'age\': 5, \'name\': \'Sam\'}\n```\nIf you create an instance of your class with an ID that isn\'t in your chosen database, then it will instantiate it using the `default` dict that you defined in the class. \n\ncaroline will automatically handle its own connections, but if you\'ve got a custom one, feel free to pass it in through your model:\n\n```python\nclass Cat(Prototype):\n    elasticsearch_conn = your_elasticsearch_connection\n    # OR\n    redis_conn = your_redis_connection\n    \n    default = {}\n```\nNOTE: You cannot have more than one connection on your model! Each model can only work with one database; that being said, you _can_ have each model route to a different database if you want to and caroline with handle it for you. If you don\'t want to pass a specific connection with each model, we don\'t blame you; The default connection is Elasticsearch, but you can change that by setting the environment variable `CAROLINE_DEFAULT_DB` to either "elasticsearch" or "redis". You can also import the caroline config directly and manually set the requested database as the default (`caroline.config.default_db = "redis"`).\n\nThere is currently not a way to change the ElasticSearch location, but you can set the Redis location by formatting your Redis address as a URL, (e.g. `redis://localhost:6379/0`) which caroline will pick up if you set it as the environment variable `CAROLINE_REDIS_URL`.\n\nIf time goes on and you need to upgrade your models, we have a plan for that! Just modify your model (add new fields or remove them), then load your keys as normal. Call `.upgrade()` on the object that you\'ve retrieved from the database and caroline will force your existing data into the new model. THIS IS A DESTRUCTIVE CALL.\n\n```python\nfrom caroline import Prototype\n\nclass Dog(Prototype):\n    default = {\n        "age": 1,\n        "name": "",\n        "sire": "",\n        "dam": ""\n    }\n    db = \'redis\'\n\nx = Dog(\'sam\')\nprint(x)\n\n# we get the last time we used the key `sam` for the prototype class `Dog`\n# >>> {\'breed\': \'mutt\', \'age\': 5, \'name\': \'Sam\'}\n\n# THIS CALL RESULTS IN THE DESTRUCTION OF DATA\nx.upgrade()\nprint(x)\n\n# >>> {\'age\': 5, \'name\': \'Sam\', \'sire\': \'\', \'dam\': \'\'}\nx.save()\n```\n\ncaroline also gives you the ability to choose your database root key names; by default, it will be the name of the class model you create. So for example, if you have a class named Dog with the record ID of \'sam\' from above, then the record in the DB will have the key of `::dog::sam`. You can adjust the first part by adding another parameter: `db_key`. Don\'t bother with this unless you have a good reason to change it.\n\nA note on schemas: while we provide support for jsonschema (and that was the driving force behind the creation of this package), you don\'t have to use it -- as the above example illustrates, you don\'t need to create a `schema` key. If no `schema` is passed in, it does not perform the validation step on `.save()` -- otherwise, it validates the data against the schema every time `.save()` is called.\n\nFinally, sometimes you just need a dict; our prototype classes do a great job of pretending to be a dict, but if you ever actually just need the data, call `.to_dict()` and a regular dictionary will be returned.\n',
    'author': 'Grafeas Group Ltd.',
    'author_email': 'devs@grafeas.org',
    'url': 'https://www.grafeas.org',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
